"""Top-level module for wtforglib Library."""

import json
from pathlib import Path
from typing import Any

import yaml

from wtforglib.errors import raise_filenotfound
from wtforglib.kinds import Fspec, StrAnyDict

WRITE_YAML_DEPRECATED = (
    "WARNING: write_yaml_file is deprecated and will be removed "
    "in a future release. Use safe_write_yaml_file."
)


def load_yaml_file(
    file_spec: Fspec,
    missing_ok: bool = True,
) -> StrAnyDict:
    """Loads a yaml file.

    Parameters
    ----------
    file_spec : Fspec
        The yaml file to load
    missing_ok : bool optional default True
        When False and file does not exist raises FileNotFound exception

    Returns
    -------
    StrAnyDict
        Representing contents of file_spec
    """
    yaml_path = Path(file_spec)
    yaml_result = {}
    if yaml_path.exists() and yaml_path.is_file():
        with open(yaml_path, "r") as yaml_file:
            yaml_result = yaml.safe_load(yaml_file)
    elif not missing_ok:  # pragma no cover
        raise_filenotfound(yaml_path)
    return yaml_result


def safe_write_yaml_file(  # type: ignore [explicit-any]
    file_spec: Fspec, src_data: StrAnyDict, **kwargs: Any
) -> bool:
    """Writes src_data to a file in a yaml format.

    Parameters
    ----------
    file_spec : Fspec
        The yaml filename to write to
    src_data : StrAnyDict
        The data to write to a file
    kwargs : Optional
        Args passed to yaml.safe_dump

    Returns
    -------
        bool
            True if file exists else False
    """
    yaml_path = Path(file_spec)
    with open(yaml_path, "w") as out_file:
        yaml.safe_dump(src_data, out_file, **kwargs)
    return yaml_path.exists()


def write_yaml_file(
    file_spec: Fspec,
    src_data: StrAnyDict,
    encoding: str = "utf-8",
) -> bool:
    """Writes src_data to a file in a yaml format.

    Parameters
    ----------
    file_spec : Fspec
        The yaml filename to write to
    src_data : StrAnyDict
        The data to write to a file
    encoding : str Optional
        encoding default 'utf-8'

    Returns
    -------
        bool
            True if file exists else False
    """
    print(WRITE_YAML_DEPRECATED)  # noqa: WPS421
    yaml_path = Path(file_spec)
    with open(yaml_path, "w") as out_file:
        yaml.dump(src_data, out_file, encoding=encoding)
    return yaml_path.exists()


def load_json_file(
    file_spec: Fspec,
    missing_ok: bool = True,
) -> StrAnyDict:
    """Loads a json file.

    Parameters
    ----------
    file_spec : Fspec
        The yaml file to load
    missing_ok : bool optional default True
        When False and file does not exist raises FileNotFound exception

    Returns
    -------
    StrAnyDict
        Representing contents of file_spec
    """
    jpath = Path(file_spec)
    rtn: StrAnyDict = {}
    if jpath.exists() and jpath.is_file():
        with open(jpath, "r") as json_file:
            rtn = json.load(json_file)
    elif not missing_ok:  # pragma no cover
        raise_filenotfound(jpath)
    return rtn


def write_json_file(
    file_spec: Fspec,
    src_data: StrAnyDict,
    indent: int = 2,
) -> bool:
    """Writes src_data to a file in a json format.

    Parameters
    ----------
    file_spec : Fspec
        The json filename to write to
    src_data : StrAnyDict
        The data to write to a file
    indent : int Optional
        The number of spaces to indent

    Returns
    -------
        bool
            True if file exists else False
    """
    jpath = Path(file_spec)
    with open(jpath, "w") as out_file:
        out_file.write(json.dumps(src_data, indent=indent))
    return jpath.exists()



"""Top-level module for wtforglib Library."""

import os
from datetime import datetime
from pathlib import Path
from tempfile import TemporaryFile
from typing import Optional, Tuple

from wtforglib.kinds import Fspec


def ensure_directory(target: Fspec, perm: int = 0o755) -> bool:
    """Creates a directory if it doesn't exist.

    Parameters
    ----------
    target : Fspec
        The path to the target directory
    perm : int Optional
        The mode to use if creation needed

    Returns
    -------
    bool
        True if directory exists

    Raises
    ------
    NotADirectoryError
        if target exists, but not a directory
    """
    dp = Path(target)
    if dp.is_dir():
        return True
    if dp.exists():
        raise NotADirectoryError(
            'Pathname "{0}" is not a directory.'.format(str(target)),
        )
    dp.mkdir(mode=perm, parents=True)
    return True


def _verify_directory_write(dpath: Fspec) -> Optional[Exception]:
    """Verify that the given directory is writable.

    Parameters
    ----------
    dpath : Fspec
        Pathlike object specifying the directory

    Returns
    -------
    Optional[Exception]
        The exception when caught
    """
    error: Optional[Exception] = None
    try:
        temp_file = TemporaryFile(dir=dpath)
        temp_file.close()
    except Exception as ex:
        error = ex
    return error


def verify_directory(dspec: Fspec, ex: bool = False) -> Tuple[bool, str]:
    """Verify that a directory exits and is writable.

    Parameters
    ----------
    dspec : Fspec
        Pathlike object specifying the directory
    ex : bool, optional
        When True exceptions are raised, by default False

    Returns
    -------
    Tuple[bool, str]
        When ex is False the status is returned

    Raises
    ------
    error
        NotADirectoryError, FileNotFoundError or other exception is raised on failure
        when ex is True
    """
    dpath = Path(dspec)
    target_path_str = str(dpath)
    error: Optional[Exception]
    if dpath.exists():
        if dpath.is_dir():
            error = _verify_directory_write(dpath)
        else:
            error = NotADirectoryError(
                "'{0}' is not a directory".format(target_path_str),
            )
    else:
        error = FileNotFoundError("Directory not found: {0}".format(dpath))
    if error:
        if ex:
            raise error
    return error is None, str(error)


def delete_empty_dirs(path: Fspec) -> int:  # noqa: WPS210
    """Delete empty directories in the given path recursively.

    Parameters
    ----------
    path : Fspec
        The given path to walk through

    Returns
    -------
    int
        Number of directories deleted
    """
    removed = 0
    for root, dirs, _files in os.walk(path, topdown=False):
        for directory in dirs:
            dirpath = os.path.join(root, directory)
            if not os.listdir(dirpath):
                os.rmdir(dirpath)
                removed += 1
    return removed


# function to perform delete operation based on condition
def prune_older_files(path: Fspec, max_days: int) -> int:  # noqa: WPS210
    """Delete files recursively in path if they are older than days.

    Parameters
    ----------
    path : Fspec
        Path to the folder to prune
    max_days : int
        Number of days to keep

    Returns
    -------
    int
        Number of files pruned
    """
    removed = 0
    # loop to check all files one by one
    # os.walk returns 3 things:
    #   - current path,
    #   - files in the current path, and
    #   - folders in the current path
    for root, _dirs, files in os.walk(path, topdown=True):
        for fn in files:
            # temp variable to store path of the file
            file_path = os.path.join(root, fn)
            # get the timestamp, when the file was modified
            timestamp_of_file_modified = os.path.getmtime(file_path)
            # convert timestamp to datetime
            modification_date = datetime.fromtimestamp(timestamp_of_file_modified)
            # find the number of days when the file was modified
            number_of_days = (datetime.now() - modification_date).days
            if number_of_days > max_days:
                # remove file
                os.remove(file_path)
                removed += 1
    return removed


def prune_older_files_empty_dir(path: Fspec, max_days: int) -> int:
    """Delete files recursively in path if they are older than days, delete empty dirs.

    Parameters
    ----------
    path : Fspec
        Path to the folder to prune
    max_days : int
        Number of days to keep

    Returns
    -------
    int
        Number of items removed
    """
    removed = prune_older_files(path, max_days)
    removed += delete_empty_dirs(path)
    return removed
